from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass
from decimal import Decimal
from typing import List

from loguru import logger

from src.core import util
from src.core.api_request import account_post
from src.core.constant import FINANCE_URL
from src.core.util import get_curr_day, get_first_day_by_str
from src.entity.account import Account
from src.entity.user import User, get_user_by_telegram_id

'''
财务报表
'''


@dataclass
class Finance(object):
    profit: Decimal
    promo: Decimal
    thirdPartySpend: Decimal
    netProfit: Decimal
    deposit: Decimal
    draw: Decimal
    rebate: Decimal
    adjust: Decimal
    netAmount: Decimal
    betAmount: Decimal
    handlingFee: Decimal
    partnershipProfit: Decimal
    name: str

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if value is not None:
                setattr(self, key, value)


def get_finance(account: Account, start_date=util.get_first_day_month(), end_date=util.get_curr_day()) -> Finance:
    """
    :rtype: Finance
    """
    api_response = account_post(url=FINANCE_URL, account=account, params={
        "startDate": start_date,
        "endDate": end_date,
        "topId": 0
    })
    finance = Finance(**api_response.data)
    finance.name = account.name
    return finance


def get_finances_by_user(user: User, date) -> List[Finance]:
    accounts = user.accounts
    start_date = util.get_first_day_by_str(date)
    with ThreadPoolExecutor(max_workers=len(accounts)) as t:
        futures = [t.submit(get_finance, account, start_date, date) for account in accounts]
        return [future.result() for future in futures]


def get_net_win_by_user(user: User, date: str) -> str:
    finances = get_finances_by_user(user, date)
    finance_strings = [f"{finance.name}: {finance.netProfit}" for finance in finances]
    logger.info(f'Finance strings: {finance_strings}')
    return '\n'.join(finance_strings)


def get_net_win_by_telegram_id(telegram_id: int) -> str:
    user = get_user_by_telegram_id(telegram_id)
    finances = get_finances_by_user(user, get_curr_day())
    finance_strings = [f"{finance.name}: {finance.netProfit}" for finance in finances]
    logger.info(f'Finance strings: {finance_strings}')
    return '\n'.join(finance_strings)


def calculate_commission(profit, employee_type, target_completion):
    # 定义提成点位
    commission_rates = {
        "1": [(0, 0.02), (100001, 0.05), (300001, 0.07), (500001, 0.08), (700001, 0.09), (1000001, 0.10)],
        "2": [(0, 0.02), (100001, 0.03), (300001, 0.04), (500001, 0.05), (700001, 0.06), (1000001, 0.07)]
    }

    # 根据负盈利选择正确的提成点位
    rates = commission_rates[employee_type]
    rate = 0
    for r in rates:
        if profit >= r[0]:
            rate = r[1]
        else:
            break

    # 计算提成
    commission = profit * target_completion * Decimal(str(rate))
    return commission


def adjust_profits(profits):
    # 新增 adjust_profits 函数处理销售额的负数抵扣逻辑
    positive_profits = [p for p in profits if p > 0]
    negative_profits = [p for p in profits if p < 0]

    for neg_profit in negative_profits:
        for i, pos_profit in enumerate(positive_profits):
            if pos_profit + neg_profit > 0:
                positive_profits[i] += neg_profit
                neg_profit = 0
                break
            else:
                neg_profit += pos_profit
                positive_profits[i] = 0

        if neg_profit != 0:
            # 当所有账户都是负数时
            return [0]

    return positive_profits + negative_profits


def calculate_adjusted_salary(user: User, date: str, employee_type):
    profits = []
    start_date = get_first_day_by_str(date)
    for account in user.accounts:
        finance = get_finance(account, start_date, date)
        print(f'{finance.name}: {finance.netProfit}')
        profits.append(int(float(finance.netProfit)))

    adjusted_profits = adjust_profits(profits)
    return calculate_salary(employee_type, adjusted_profits)


def calculate_salary(employee_type, agent_profit):
    if employee_type == "2":
        base_salary = 10000
    else:
        base_salary = 12000
    total_salary = 0
    for profit in agent_profit:
        total_salary += calculate_commission(profit, employee_type, 1)
    if total_salary > 70000:
        base_salary = 0
    return total_salary + base_salary


def get_adjusted_salary(user: User, date: str):
    return f'方式一：{calculate_adjusted_salary(user, date, "1")}\n方式二：{calculate_adjusted_salary(user, date, "2")}'
